// Game variables
let canvas: HTMLCanvasElement;
let ctx: CanvasRenderingContext2D;
let scoreEl: HTMLElement | null;
let speedEl: HTMLElement | null;
let gameOverEl: HTMLElement | null;
let finalScoreEl: HTMLElement | null;
let gameLoop: number;
let score = 0;
const gameSpeed = 5;
let gameOver = false;
let gameStarted = false;
const devOrientationMode = false; // Developer mode for orientation testing

// Game constants
let ROAD_WIDTH = 800; // Default width
let ROAD_HEIGHT = 600; // Default height
let CAR_WIDTH = 80; // Default width
let CAR_HEIGHT = 160; // Default height
const LANE_COUNT = 3;
let LANE_WIDTH = ROAD_WIDTH / LANE_COUNT;

// Add car image variable
let carImage: HTMLImageElement;

// Add after game variables
let collisionParticles: Particle[] = [];
const screenShake = { x: 0, y: 0, intensity: 0 };
const shockwaves: Shockwave[] = [];

// Add countdown variables after game variables
let isCountingDown = false;
let countdownValue = 3;
let countdownInterval: number;

// Add particle interface
interface Particle {
    x: number;
    y: number;
    vx: number;
    vy: number;
    size: number;
    color: string;
    alpha: number;
    life: number;
}

// Add shockwave interface
interface Shockwave {
    x: number;
    y: number;
    radius: number;
    maxRadius: number;
    alpha: number;
}

// Update vehicle type definition
type VehicleType = 'SPORTS' | 'PICKUP' | 'POLICE' | 'BMW' | 'BIKE';

// Update CAR_TYPES with type
const CAR_TYPES: Record<VehicleType, string> = {
    SPORTS: '/components/games/CarRacing/Travel-car-topview.svg',
    PICKUP: '/components/games/CarRacing/pickup.svg',
    POLICE: '/components/games/CarRacing/police.svg',
    BMW: '/components/games/CarRacing/bmw.svg',
    BIKE: '/components/games/CarRacing/bike.svg'
};

// Update vehicle orientation configuration with type
const VEHICLE_ORIENTATIONS: Record<VehicleType, number> = {
    SPORTS: -Math.PI / 2, 
    PICKUP: -Math.PI / 2,
    POLICE: 50.3, // Changed to face up (270 degrees)
    BMW: Math.PI / 2, 
    BIKE: 50.3 // Changed to face up (270 degrees)
};

// Add vehicle-specific scaling
const VEHICLE_SCALES: Record<VehicleType, number> = {
    SPORTS: 1,
    PICKUP: 1,
    POLICE: 0.4, // Even smaller scale for police car
    BMW: 1,
    BIKE: 0.5  // Keep bike small
};

// Add car images cache
const carImages: { [key: string]: HTMLImageElement } = {};

// Add after game variables
const DIFFICULTY_LEVELS = {
    BEGINNER: { maxSpeed: 6, acceleration: 0.08, trafficSpeed: 0.8, trafficFrequency: 600 },
    NOVICE: { maxSpeed: 7, acceleration: 0.1, trafficSpeed: 1, trafficFrequency: 550 },
    EASY: { maxSpeed: 8, acceleration: 0.12, trafficSpeed: 1.2, trafficFrequency: 500 },
    CASUAL: { maxSpeed: 9, acceleration: 0.15, trafficSpeed: 1.5, trafficFrequency: 450 },
    MEDIUM: { maxSpeed: 10, acceleration: 0.18, trafficSpeed: 1.8, trafficFrequency: 400 },
    CHALLENGING: { maxSpeed: 11, acceleration: 0.2, trafficSpeed: 2, trafficFrequency: 350 },
    HARD: { maxSpeed: 12, acceleration: 0.22, trafficSpeed: 2.2, trafficFrequency: 300 },
    EXPERT: { maxSpeed: 13, acceleration: 0.25, trafficSpeed: 2.5, trafficFrequency: 250 }
};

let currentDifficulty = DIFFICULTY_LEVELS.BEGINNER;

// Function to calculate dimensions
function calculateDimensions() {
    if (typeof window !== 'undefined') {
        // Make road width 80% of window width
        ROAD_WIDTH = Math.min(window.innerWidth * 0.8, 1200); // Cap max width at 1200px
        // Height should be full viewport height
        ROAD_HEIGHT = window.innerHeight;
        // Car size proportional to road width
        CAR_WIDTH = ROAD_WIDTH * 0.1; // 10% of road width
        CAR_HEIGHT = CAR_WIDTH * 2; // Maintain car aspect ratio
        LANE_WIDTH = ROAD_WIDTH / LANE_COUNT;
    }
}

// Update Colors with new theme
const COLORS = {
    ROAD: '#1a1a2e', // Darker asphalt for more contrast
    ROAD_BORDER: '#16213e', // Deeper edge color
    GRASS: '#38b000', // Vibrant grass
    GRASS_PATTERN: '#008000', // Darker grass for pattern
    LINES: '#ffd60a', // Brighter yellow lines
    PLAYER_CAR: '#E74C3C', // Bright red for player
    TRAFFIC_CARS: ['#3498DB', '#9B59B6', '#1ABC9C', '#F39C12'], // Vibrant car colors
    ROAD_MARKS: '#ffffff', // Brighter white road markings
    START_LINE: '#ffffff',
    FINISH_LINE: '#ffffff',
    CHECKER: '#000000'
};

// Game state
let currentSpeed = 0;
let distance = 0;
let lastTrafficSpawn = 0;

// Export necessary functions and variables
export {
    score,
    gameSpeed,
    gameOver,
    gameStarted
};

// Player car
const playerCar = {
    x: 0,
    y: 0,
    width: 0,
    height: 0,
    speed: 0,
    maxSpeed: DIFFICULTY_LEVELS.MEDIUM.maxSpeed,
    acceleration: DIFFICULTY_LEVELS.MEDIUM.acceleration,
    deceleration: 0.1,
    handling: 5,
    currentLane: 1,
    label: 'PLAYER'
};

// Update TrafficCar interface
interface TrafficCar {
    x: number;
    y: number;
    width: number;
    height: number;
    speed: number;
    color: string;
    carType: VehicleType;
}

let trafficCars: TrafficCar[] = [];

// Environment elements
interface RoadLine {
    x: number;
    y: number;
    width: number;
    height: number;
}

const roadLines: RoadLine[] = [];

// Add after RoadLine interface
interface Tree {
    x: number;
    y: number;
    size: number;
    swayOffset: number;
    swaySpeed: number;
    type: 'pine' | 'oak';
}

// Add after roadLines array
const trees: Tree[] = [];

// Add road pattern interface
interface RoadPattern {
    x: number;
    y: number;
    width: number;
    height: number;
    type: 'chevron' | 'dot' | 'stripe';
}

// Add road patterns array
const roadPatterns: RoadPattern[] = [];

// Add after RoadPattern interface
interface RaceLine {
    y: number;
    isStart: boolean;
    pattern: boolean[][];
    animationOffset: number;
}

// Add after roadPatterns array
const startLine: RaceLine = {
    y: ROAD_HEIGHT - 100,
    isStart: true,
    pattern: Array(8).fill(null).map((_, i) => 
        Array(8).fill(null).map((_, j) => (i + j) % 2 === 0)
    ),
    animationOffset: 0
};

// Update finish line initial position
const finishLine: RaceLine = {
    y: -10000, // Much longer distance for extended gameplay
    isStart: false,
    pattern: Array(8).fill(null).map((_, i) => 
        Array(8).fill(null).map((_, j) => (i + j) % 2 === 0)
    ),
    animationOffset: 0
};

// Preload car images
function preloadCarImages() {
    Object.entries(CAR_TYPES).forEach(([key, path]) => {
        const img = new Image();
        img.src = path;
        carImages[key] = img;
    });
}

// Initialize game function
export function initGame(
    canvasElement: HTMLCanvasElement,
    scoreElement: HTMLElement,
    speedElement: HTMLElement,
    gameOverElement: HTMLElement,
    finalScoreElement: HTMLElement
) {
    canvas = canvasElement;
    ctx = canvas.getContext('2d')!;
    scoreEl = scoreElement;
    speedEl = speedElement;
    gameOverEl = gameOverElement;
    finalScoreEl = finalScoreElement;

    // Prevent scrolling when arrow keys are pressed
    window.addEventListener('keydown', (e) => {
        if([' ', 'ArrowUp', 'ArrowDown', 'ArrowLeft', 'ArrowRight'].includes(e.key)) {
            e.preventDefault();
        }
    });

    // Prevent scrolling on mobile
    canvas.addEventListener('touchmove', (e) => {
        e.preventDefault();
    }, { passive: false });

    // Preload all car images
    preloadCarImages();

    // Load player car image
    carImage = new Image();
    carImage.src = CAR_TYPES.SPORTS;
    carImage.onload = () => {
        calculateDimensions();
        canvas.width = ROAD_WIDTH;
        canvas.height = ROAD_HEIGHT;
        
        playerCar.width = CAR_WIDTH;
        playerCar.height = CAR_HEIGHT;
        playerCar.x = ROAD_WIDTH / 2 - CAR_WIDTH / 2;
        playerCar.y = ROAD_HEIGHT - CAR_HEIGHT - 20;

        initRoadLines();
        startLine.y = ROAD_HEIGHT - 100;
        finishLine.y = -10000;
        setDifficulty('BEGINNER');

        if (devOrientationMode) {
            // In dev mode, spawn static cars and start immediately
            spawnDevOrientationCars();
            gameStarted = true;
            isCountingDown = false;
            currentSpeed = 0;
            gameLoop = window.setInterval(update, 1000/60);
        } else {
            // Normal game start with countdown
            startCountdown();
        }
    };
}

// Handle window resize
function handleResize() {
    // Store current lane
    const currentLane = playerCar.currentLane;
    
    // Recalculate dimensions
    calculateDimensions();
    
    // Update canvas size
    canvas.width = ROAD_WIDTH;
    canvas.height = ROAD_HEIGHT;
    
    // Update player car dimensions and position
    playerCar.width = CAR_WIDTH;
    playerCar.height = CAR_HEIGHT;
    playerCar.x = currentLane * LANE_WIDTH + (LANE_WIDTH - CAR_WIDTH) / 2;
    playerCar.y = ROAD_HEIGHT - CAR_HEIGHT - 20;
    
    // Update traffic cars dimensions
    trafficCars.forEach(car => {
        car.width = CAR_WIDTH;
        car.height = CAR_HEIGHT;
        const lane = Math.floor(car.x / (ROAD_WIDTH / LANE_COUNT));
        car.x = lane * LANE_WIDTH + (LANE_WIDTH - CAR_WIDTH) / 2;
    });
    
    // Reinitialize road lines
    roadLines.length = 0;
    trees.length = 0;
    roadPatterns.length = 0;
    initRoadLines();
}

// Update initRoadLines function to include trees and patterns
function initRoadLines() {
    const lineCount = 10;
    const lineSpacing = ROAD_HEIGHT / lineCount;
    
    // Clear existing elements
    roadLines.length = 0;
    trees.length = 0;
    roadPatterns.length = 0;
    
    // Add lane lines
    for (let i = 0; i < lineCount; i++) {
        // Left lane line
        roadLines.push({
            x: ROAD_WIDTH / 3,
            y: i * lineSpacing,
            width: 5,
            height: 30
        });
        
        // Right lane line
        roadLines.push({
            x: (ROAD_WIDTH / 3) * 2,
            y: i * lineSpacing,
            width: 5,
            height: 30
        });

        // Add chevron patterns
        roadPatterns.push({
            x: ROAD_WIDTH / 2,
            y: i * lineSpacing,
            width: 40,
            height: 20,
            type: 'chevron'
        });

        // Add side stripes
        roadPatterns.push({
            x: 10,
            y: i * lineSpacing,
            width: 5,
            height: 40,
            type: 'stripe'
        });
        roadPatterns.push({
            x: ROAD_WIDTH - 15,
            y: i * lineSpacing,
            width: 5,
            height: 40,
            type: 'stripe'
        });
    }

    // Add dots between lanes
    const dotSpacing = 100;
    for (let i = 0; i < ROAD_HEIGHT / dotSpacing; i++) {
        roadPatterns.push({
            x: ROAD_WIDTH / 3 - 15,
            y: i * dotSpacing,
            width: 4,
            height: 4,
            type: 'dot'
        });
        roadPatterns.push({
            x: (ROAD_WIDTH / 3) * 2 + 15,
            y: i * dotSpacing,
            width: 4,
            height: 4,
            type: 'dot'
        });
    }

    // Initialize trees on both sides
    const treeCount = 15;
    const treeSpacing = ROAD_HEIGHT / treeCount;
    const treeTypes: ('pine' | 'oak')[] = ['pine', 'oak'];
    
    for (let i = 0; i < treeCount; i++) {
        // Left side trees
        trees.push({
            x: -50, // Left of the road
            y: i * treeSpacing,
            size: 30 + Math.random() * 20,
            swayOffset: Math.random() * Math.PI * 2,
            swaySpeed: 0.5 + Math.random() * 0.5,
            type: treeTypes[Math.floor(Math.random() * treeTypes.length)]
        });
        
        // Right side trees
        trees.push({
            x: ROAD_WIDTH + 20, // Right of the road
            y: i * treeSpacing + treeSpacing / 2, // Offset for variety
            size: 30 + Math.random() * 20,
            swayOffset: Math.random() * Math.PI * 2,
            swaySpeed: 0.5 + Math.random() * 0.5,
            type: treeTypes[Math.floor(Math.random() * treeTypes.length)]
        });
    }
}

// Game update function
function update() {
    if (gameOver) return;

    ctx.save();
    
    // Clear canvas
    ctx.clearRect(0, 0, canvas.width, canvas.height);

    // Update game elements
    updateRoad();
    updateTrees();
    
    // Only update game elements if game has started and not in dev mode
    if (gameStarted && !devOrientationMode) {
        updatePlayer();
        updateTraffic();
        checkCollisions();
        updateScore();
        
        if (!isCountingDown) {
            startLine.y += currentSpeed;
            finishLine.y += currentSpeed;
            
            if (playerCar.y < finishLine.y + 50 && playerCar.y > finishLine.y - 50) {
                handleVictory();
            }
            
            const distanceToFinish = Math.max(0, Math.abs(finishLine.y - playerCar.y));
            if (speedEl) {
                const distanceInKm = (distanceToFinish/1000).toFixed(1);
                speedEl.textContent = `${Math.floor(currentSpeed * 10)} Km/h | Finish: ${distanceInKm}km`;
            }
        }
    }

    // Draw game elements
    drawEnvironment();
    drawPlayer();
    drawTraffic();
    drawParticles();
    drawShockwaves();
    drawUI();

    // Draw countdown if active and not in dev mode
    if (isCountingDown && !devOrientationMode) {
        drawCountdown();
    }

    ctx.restore();
}

// Export reset game function
export function resetGame() {
    collisionParticles = [];
    score = 0;
    gameOver = false;
    currentSpeed = 0;
    distance = 0;
    trafficCars = [];
    
    if (scoreEl) scoreEl.textContent = '0';
    if (speedEl) speedEl.textContent = '0 Km/h';
    
    playerCar.x = ROAD_WIDTH / 2 - CAR_WIDTH / 2;
    playerCar.y = ROAD_HEIGHT - CAR_HEIGHT - 20;
    playerCar.speed = 0;
    playerCar.currentLane = 1;
    
    if (gameOverEl) {
        gameOverEl.classList.add('hidden');
    }
    
    initRoadLines();
    gameLoop = window.setInterval(update, 1000/60);
    gameStarted = true;
}

// Update road function
function updateRoad() {
    // Update road lines with speed-based spacing
    roadLines.forEach(line => {
        line.y += currentSpeed * 1.2; // Increased speed effect
        if (line.y > ROAD_HEIGHT) {
            line.y = -line.height;
        }
    });

    // Add speed lines effect when going fast
    if (currentSpeed > gameSpeed * 0.8) {
        ctx.save();
        ctx.strokeStyle = 'rgba(255, 255, 255, 0.1)';
        ctx.lineWidth = 2;
        for (let i = 0; i < 10; i++) {
            const y = (Date.now() / (10 + i * 10)) % ROAD_HEIGHT;
            ctx.beginPath();
            ctx.moveTo(0, y);
            ctx.lineTo(ROAD_WIDTH, y);
            ctx.stroke();
        }
        ctx.restore();
    }
}

// Update player function
function updatePlayer() {
    // Update speed
    if (currentSpeed < playerCar.maxSpeed) {
        currentSpeed += playerCar.acceleration;
    }
    
    // Update distance and score
    distance += currentSpeed;
    score = Math.floor(distance / 100);
    
    // Update speed display
    if (speedEl) {
        speedEl.textContent = `${Math.floor(currentSpeed * 10)} Km/h`;
    }

    if (currentSpeed > gameSpeed * 0.7) {
        // Add speed particles
        ctx.save();
        ctx.fillStyle = 'rgba(255, 255, 255, 0.3)';
        for (let i = 0; i < currentSpeed/2; i++) {
            const particleX = playerCar.x + Math.random() * CAR_WIDTH;
            const particleY = playerCar.y + CAR_HEIGHT + Math.random() * 20;
            const size = Math.random() * 3;
            ctx.fillRect(particleX, particleY, size, size);
        }
        ctx.restore();
    }
}

// Update traffic function
function updateTraffic() {
    if (devOrientationMode) return; // Don't update traffic in dev mode
    
    // Only spawn new cars if there's enough space from the last car
    const lastCar = trafficCars[trafficCars.length - 1];
    const minSpawnDistance = currentDifficulty.trafficFrequency;
    
    if (!lastCar || (lastCar && lastCar.y > 0)) {
        if (distance - lastTrafficSpawn > minSpawnDistance) {
            spawnTrafficCar();
            lastTrafficSpawn = distance;
        }
    }
    
    // Update existing traffic cars
    trafficCars.forEach((car, index) => {
        car.y += (currentSpeed - car.speed);
        
        // Remove cars that are off screen
        if (car.y > ROAD_HEIGHT) {
            trafficCars.splice(index, 1);
        }
    });
}

// Spawn traffic car function
function spawnTrafficCar() {
    const lane = Math.floor(Math.random() * LANE_COUNT);
    
    // Check if there's enough space in the chosen lane
    const canSpawnInLane = !trafficCars.some(car => 
        car.y < 0 && car.y > -CAR_HEIGHT * 4 && 
        Math.abs(car.x - (lane * LANE_WIDTH + (LANE_WIDTH - CAR_WIDTH) / 2)) < LANE_WIDTH
    );
    
    if (!canSpawnInLane) {
        return; // Don't spawn if there's not enough space
    }
    
    const color = COLORS.TRAFFIC_CARS[Math.floor(Math.random() * COLORS.TRAFFIC_CARS.length)];
    
    // Randomly select vehicle type with different probabilities
    const vehicleRoll = Math.random();
    let carType: VehicleType;
    if (vehicleRoll < 0.3) {
        carType = 'SPORTS';
    } else if (vehicleRoll < 0.5) {
        carType = 'PICKUP';
    } else if (vehicleRoll < 0.7) {
        carType = 'BMW';
    } else if (vehicleRoll < 0.9) {
        carType = 'POLICE';
    } else {
        carType = 'BIKE';
    }
    
    // Randomize vertical spacing a bit
    const verticalOffset = Math.random() * CAR_HEIGHT * 2;
    
    trafficCars.push({
        x: lane * LANE_WIDTH + (LANE_WIDTH - CAR_WIDTH) / 2,
        y: -CAR_HEIGHT - verticalOffset,
        width: CAR_WIDTH,
        height: CAR_HEIGHT,
        speed: Math.random() * 1.5 + currentDifficulty.trafficSpeed,
        color,
        carType
    });
}

// Check collisions function
function checkCollisions() {
    trafficCars.forEach(car => {
        if (isColliding(playerCar, car)) {
            handleGameOver();
        }
    });
}

// Collision detection helper
interface GameObject {
    x: number;
    y: number;
    width: number;
    height: number;
}

// Update hitbox calculation helper
function getCarHitbox(car: GameObject) {
    // Create a very precise hitbox (95% of the car size)
    const hitboxScale = 0.95;
    const hitboxWidth = car.width * hitboxScale;
    const hitboxHeight = car.height * 0.5; // Make height hitbox more precise
    
    return {
        x: car.x + (car.width - hitboxWidth) / 2,
        y: car.y + car.height * 0.25, // Position hitbox in the middle section of the car
        width: hitboxWidth,
        height: hitboxHeight
    };
}

// Update collision detection helper
function isColliding(rect1: GameObject, rect2: GameObject) {
    // Get hitboxes for both cars
    const hitbox1 = getCarHitbox(rect1);
    const hitbox2 = getCarHitbox(rect2);

    // Precise collision check without buffer
    return hitbox1.x < (hitbox2.x + hitbox2.width) &&
           (hitbox1.x + hitbox1.width) > hitbox2.x &&
           hitbox1.y < (hitbox2.y + hitbox2.height) &&
           (hitbox1.y + hitbox1.height) > hitbox2.y;
}

// Update draw hitbox for debugging

// Update score function
function updateScore() {
    if (scoreEl) {
        scoreEl.textContent = score.toString();
    }
}

// Draw environment function
function drawEnvironment() {
    // Draw grass with pattern
    ctx.fillStyle = COLORS.GRASS;
    ctx.fillRect(0, 0, canvas.width, canvas.height);
    
    // Add grass pattern
    for (let x = 0; x < canvas.width; x += 20) {
        for (let y = 0; y < canvas.height; y += 20) {
            if ((x + y) % 40 === 0) {
                ctx.fillStyle = COLORS.GRASS_PATTERN;
                ctx.fillRect(x, y, 10, 10);
            }
        }
    }
    
    // Draw road with gradient
    const roadGradient = ctx.createLinearGradient(0, 0, ROAD_WIDTH, 0);
    roadGradient.addColorStop(0, COLORS.ROAD_BORDER);
    roadGradient.addColorStop(0.1, COLORS.ROAD);
    roadGradient.addColorStop(0.9, COLORS.ROAD);
    roadGradient.addColorStop(1, COLORS.ROAD_BORDER);
    ctx.fillStyle = roadGradient;
    ctx.fillRect(0, 0, ROAD_WIDTH, ROAD_HEIGHT);
    
    // Draw road patterns
    roadPatterns.forEach(pattern => {
        pattern.y += currentSpeed;
        if (pattern.y > ROAD_HEIGHT) {
            pattern.y = -pattern.height;
        }
        
        ctx.fillStyle = COLORS.ROAD_MARKS;
        
        switch (pattern.type) {
            case 'chevron':
                drawChevron(pattern);
                break;
            case 'dot':
                ctx.beginPath();
                ctx.arc(pattern.x + pattern.width/2, pattern.y + pattern.height/2, 
                       pattern.width/2, 0, Math.PI * 2);
                ctx.fill();
                break;
            case 'stripe':
                ctx.fillRect(pattern.x, pattern.y, pattern.width, pattern.height);
                break;
        }
    });
    
    // Draw trees
    trees.forEach(drawTree);
    
    // Draw road lines
    ctx.fillStyle = COLORS.LINES;
    roadLines.forEach(line => {
        ctx.fillRect(line.x, line.y, line.width, line.height);
    });

    // Draw race lines
    drawRaceLine(startLine);
    drawRaceLine(finishLine);
}

// Draw player function
function drawPlayer() {
    if (currentSpeed > gameSpeed * 0.7) {
        // Add speed particles
        ctx.save();
        ctx.fillStyle = 'rgba(255, 255, 255, 0.3)';
        for (let i = 0; i < currentSpeed/2; i++) {
            const particleX = playerCar.x + Math.random() * CAR_WIDTH;
            const particleY = playerCar.y + CAR_HEIGHT + Math.random() * 20;
            const size = Math.random() * 3;
            ctx.fillRect(particleX, particleY, size, size);
        }
        ctx.restore();
    }
    
    drawCar(playerCar.x, playerCar.y, COLORS.PLAYER_CAR, 'SPORTS', playerCar.label);
}

// Draw traffic function
function drawTraffic() {
    trafficCars.forEach(car => {
        drawCar(car.x, car.y, car.color, car.carType);
        // Uncomment the next line to see hitboxes during development
        // drawHitbox(car);
    });
}

// Draw car helper function
function drawCar(x: number, y: number, color: string, carType: VehicleType = 'SPORTS', label?: string) {
    const image = carImages[carType];
    if (image && image.complete) {
        ctx.save();
        ctx.translate(x + CAR_WIDTH / 2, y + CAR_HEIGHT / 2);
        
        // Apply vehicle-specific rotation
        const rotation = VEHICLE_ORIENTATIONS[carType] || -Math.PI / 2;
        ctx.rotate(rotation);
        
        // Apply vehicle-specific scaling
        const vehicleScale = VEHICLE_SCALES[carType] || 1;
        const baseScale = CAR_WIDTH / image.width;
        ctx.scale(baseScale * vehicleScale, baseScale * vehicleScale);
        
        ctx.drawImage(
            image,
            -image.width / 2,
            -image.height / 2,
            image.width,
            image.height
        );
        
        // Enhanced label drawing - now below the car
        if (label) {
            ctx.restore(); // Restore to remove rotation
            ctx.save(); // Save new state
            
            // Create gradient for label
            const gradient = ctx.createLinearGradient(x, y + CAR_HEIGHT + 5, x + CAR_WIDTH, y + CAR_HEIGHT + 20);
            gradient.addColorStop(0, '#FFD700'); // Gold
            gradient.addColorStop(0.5, '#FFA500'); // Orange
            gradient.addColorStop(1, '#FF4500'); // Red-Orange
            
            // Draw label with glow effect - smaller font
            ctx.font = 'bold 16px Arial';
            ctx.textAlign = 'center';
            ctx.textBaseline = 'top';
            
            // Add glow effect
            ctx.shadowColor = '#FF8C00';
            ctx.shadowBlur = 8;
            ctx.shadowOffsetX = 0;
            ctx.shadowOffsetY = 0;
            
            // Draw outline
            ctx.strokeStyle = '#000000';
            ctx.lineWidth = 2;
            ctx.strokeText(label, x + CAR_WIDTH/2, y + CAR_HEIGHT + 5);
            
            // Draw text with gradient
            ctx.fillStyle = gradient;
            ctx.fillText(label, x + CAR_WIDTH/2, y + CAR_HEIGHT + 5);
        } else {
            ctx.restore(); // Restore if no label
        }
        return;
    }
    
    // Fallback rectangle with enhanced label
    ctx.fillStyle = color;
    ctx.fillRect(x, y, CAR_WIDTH, CAR_HEIGHT);
    
    if (label) {
        // Create gradient for label
        const gradient = ctx.createLinearGradient(x, y + CAR_HEIGHT + 5, x + CAR_WIDTH, y + CAR_HEIGHT + 20);
        gradient.addColorStop(0, '#FFD700');
        gradient.addColorStop(0.5, '#FFA500');
        gradient.addColorStop(1, '#FF4500');
        
        ctx.font = 'bold 16px Arial';
        ctx.textAlign = 'center';
        ctx.textBaseline = 'top';
        
        // Add glow effect
        ctx.shadowColor = '#FF8C00';
        ctx.shadowBlur = 8;
        ctx.shadowOffsetX = 0;
        ctx.shadowOffsetY = 0;
        
        // Draw outline
        ctx.strokeStyle = '#000000';
        ctx.lineWidth = 2;
        ctx.strokeText(label, x + CAR_WIDTH/2, y + CAR_HEIGHT + 5);
        
        // Draw text with gradient
        ctx.fillStyle = gradient;
        ctx.fillText(label, x + CAR_WIDTH/2, y + CAR_HEIGHT + 5);
    }
}

// Draw UI function
function drawUI() {
    // Score and speed are handled by HTML elements
}

// Handle game over function
function handleGameOver() {
    gameOver = true;
    createExplosion(playerCar.x, playerCar.y);
    
    setTimeout(() => {
        clearInterval(gameLoop);
        gameStarted = false;
        
        if (gameOverEl) {
            gameOverEl.classList.remove('hidden');
            gameOverEl.innerHTML = `
                <h2>Game Over</h2>
                <p>Final Score: ${score}</p>
                <button id="playAgainBtn">Play Again</button>
            `;
            
            // Add event listener to the play again button
            const playAgainBtn = document.getElementById('playAgainBtn');
            if (playAgainBtn) {
                playAgainBtn.addEventListener('click', () => {
                    resetGame();
                    gameOverEl?.classList.add('hidden');
                });
            }
        }
        
        if (finalScoreEl) {
            finalScoreEl.textContent = score.toString();
        }
    }, 1000);
}

// Add control instructions interface
interface ControlInstruction {
    key: string;
    action: string;
    icon: string;
}

// Update control instructions with friendlier text
const CONTROL_INSTRUCTIONS: ControlInstruction[] = [
    { key: 'A / ←', action: 'Steer Left', icon: '🚗' },
    { key: 'D / →', action: 'Steer Right', icon: '🚗' },
    { key: '↑', action: 'Accelerate', icon: '🏎️' },
    { key: '↓', action: 'Brake', icon: '🛑' }
];

// Update handleKeyDown function to prevent default behavior
export function handleKeyDown(event: KeyboardEvent) {
    if (devOrientationMode) return; // Disable controls in dev mode
    
    if (gameOver || !gameStarted || isCountingDown) return;
    
    // Prevent default behavior for game controls
    if(['ArrowUp', 'ArrowDown', 'ArrowLeft', 'ArrowRight', 'a', 'd'].includes(event.key.toLowerCase())) {
        event.preventDefault();
    }
    
    const laneWidth = ROAD_WIDTH / LANE_COUNT;
    
    switch (event.key.toLowerCase()) {
        case 'arrowleft':
        case 'a':
            if (playerCar.currentLane > 0) {
                playerCar.currentLane--;
                playerCar.x = playerCar.currentLane * laneWidth + (laneWidth - CAR_WIDTH) / 2;
            }
            break;
        case 'arrowright':
        case 'd':
            if (playerCar.currentLane < LANE_COUNT - 1) {
                playerCar.currentLane++;
                playerCar.x = playerCar.currentLane * laneWidth + (laneWidth - CAR_WIDTH) / 2;
            }
            break;
        case 'arrowup':
            playerCar.speed = Math.min(playerCar.maxSpeed, playerCar.speed + playerCar.acceleration * 3);
            currentSpeed = playerCar.speed;
            break;
        case 'arrowdown':
            playerCar.speed = Math.max(0, playerCar.speed - playerCar.deceleration * 3);
            currentSpeed = playerCar.speed;
            break;
    }
}

// Export cleanup function
export function cleanup() {
    window.removeEventListener('resize', handleResize);
    clearInterval(gameLoop);
}

// Update collision animation function
function createExplosion(x: number, y: number) {
    // Create more particles with varied effects
    const particleCount = 60; // Increased from 30
    const colors = ['#FF0000', '#FF5500', '#FFAA00', '#FFFF00', '#FFFFFF'];
    
    // Initial burst particles
    for (let i = 0; i < particleCount; i++) {
        const angle = (Math.PI * 2 / particleCount) * i;
        const speed = 3 + Math.random() * 5; // Increased speed
        
        collisionParticles.push({
            x: x + CAR_WIDTH / 2,
            y: y + CAR_HEIGHT / 2,
            vx: Math.cos(angle) * speed,
            vy: Math.sin(angle) * speed,
            size: 8 + Math.random() * 8, // Larger particles
            color: colors[Math.floor(Math.random() * colors.length)],
            alpha: 1,
            life: 80 // Longer life
        });
    }

    // Add secondary smaller particles
    for (let i = 0; i < particleCount/2; i++) {
        const angle = Math.random() * Math.PI * 2;
        const speed = 1 + Math.random() * 2;
        
        collisionParticles.push({
            x: x + CAR_WIDTH / 2,
            y: y + CAR_HEIGHT / 2,
            vx: Math.cos(angle) * speed,
            vy: Math.sin(angle) * speed,
            size: 2 + Math.random() * 3,
            color: '#FFFFFF',
            alpha: 1,
            life: 120
        });
    }

    // Add shockwave
    shockwaves.push({
        x: x + CAR_WIDTH / 2,
        y: y + CAR_HEIGHT / 2,
        radius: 10,
        maxRadius: 150,
        alpha: 1
    });

    // Add screen shake
    screenShake.intensity = 20;
}

// Update particles function

// Draw particles function
function drawParticles() {
    collisionParticles.forEach(particle => {
        ctx.save();
        ctx.globalAlpha = particle.alpha;
        ctx.fillStyle = particle.color;
        
        // Add glow effect
        ctx.shadowColor = particle.color;
        ctx.shadowBlur = 10;
        
        ctx.beginPath();
        ctx.arc(particle.x, particle.y, particle.size, 0, Math.PI * 2);
        ctx.fill();
        ctx.restore();
    });
}

// Add shockwave update function

// Add shockwave draw function
function drawShockwaves() {
    shockwaves.forEach(wave => {
        ctx.save();
        ctx.strokeStyle = `rgba(255, 255, 255, ${wave.alpha})`;
        ctx.lineWidth = 3;
        ctx.beginPath();
        ctx.arc(wave.x, wave.y, wave.radius, 0, Math.PI * 2);
        ctx.stroke();
        
        // Add inner glow
        const gradient = ctx.createRadialGradient(
            wave.x, wave.y, wave.radius * 0.5,
            wave.x, wave.y, wave.radius
        );
        gradient.addColorStop(0, `rgba(255, 100, 0, 0)`);
        gradient.addColorStop(1, `rgba(255, 50, 0, ${wave.alpha * 0.3})`);
        ctx.fillStyle = gradient;
        ctx.fill();
        ctx.restore();
    });
}

// Update set difficulty function
export function setDifficulty(level: 'BEGINNER' | 'NOVICE' | 'EASY' | 'CASUAL' | 'MEDIUM' | 'CHALLENGING' | 'HARD' | 'EXPERT') {
    currentDifficulty = DIFFICULTY_LEVELS[level];
    playerCar.maxSpeed = currentDifficulty.maxSpeed;
    playerCar.acceleration = currentDifficulty.acceleration;
    
    // Update player car label to show only "PLAYER"
    playerCar.label = 'PLAYER';
}

// Add updateTrees function
function updateTrees() {
    trees.forEach(tree => {
        tree.y += currentSpeed * 1.1; // Slightly faster tree movement
        if (tree.y > ROAD_HEIGHT) {
            tree.y = -tree.size;
        }
        // More pronounced sway based on speed
        tree.swayOffset += (tree.swaySpeed * 0.02) * (1 + currentSpeed/gameSpeed/2);
    });
}

// Add drawTree function
function drawTree(tree: Tree) {
    const sway = Math.sin(tree.swayOffset) * 5; // Sway amount
    
    ctx.save();
    ctx.translate(tree.x + sway, tree.y);
    
    if (tree.type === 'pine') {
        // Draw pine tree
        ctx.fillStyle = '#2d5a27'; // Dark green for pine
        // Tree triangles
        for (let i = 0; i < 3; i++) {
            ctx.beginPath();
            ctx.moveTo(-tree.size/2, -i * tree.size/3);
            ctx.lineTo(tree.size/2, -i * tree.size/3);
            ctx.lineTo(0, -tree.size/2 - i * tree.size/3);
            ctx.closePath();
            ctx.fill();
        }
        // Trunk
        ctx.fillStyle = '#4a2f1b';
        ctx.fillRect(-tree.size/8, 0, tree.size/4, tree.size/2);
    } else {
        // Draw oak tree
        ctx.fillStyle = '#4a2f1b';
        ctx.fillRect(-tree.size/8, 0, tree.size/4, tree.size/2); // Trunk
        
        // Leafy top
        ctx.beginPath();
        ctx.fillStyle = '#3d7a3d';
        ctx.arc(0, -tree.size/4, tree.size/2, 0, Math.PI * 2);
        ctx.fill();
    }
    
    ctx.restore();
}

// Add chevron drawing function
function drawChevron(pattern: RoadPattern) {
    ctx.beginPath();
    ctx.moveTo(pattern.x, pattern.y + pattern.height);
    ctx.lineTo(pattern.x + pattern.width/2, pattern.y);
    ctx.lineTo(pattern.x + pattern.width, pattern.y + pattern.height);
    ctx.strokeStyle = COLORS.ROAD_MARKS;
    ctx.lineWidth = 3;
    ctx.stroke();
}

// Update drawRaceLine function to handle finish line
function drawRaceLine(line: RaceLine) {
    const tileSize = ROAD_WIDTH / line.pattern[0].length;
    const totalHeight = line.isStart ? tileSize * line.pattern.length * 2 : tileSize * line.pattern.length;
    
    if (gameStarted && !isCountingDown) {
        // Faster animation for more excitement
        line.animationOffset = (line.animationOffset + currentSpeed * 0.15) % tileSize;
    }
    
    ctx.save();
    
    if (line.isStart) {
        // Enhanced start line effect
        const gradient = ctx.createLinearGradient(0, line.y, 0, line.y + totalHeight);
        gradient.addColorStop(0, 'rgba(255, 255, 255, 0.8)');
        gradient.addColorStop(0.5, 'rgba(255, 255, 255, 1)');
        gradient.addColorStop(1, 'rgba(255, 255, 255, 0.8)');
        
        ctx.fillStyle = gradient;
        ctx.shadowColor = '#00FF00';
        ctx.shadowBlur = 20 + Math.sin(Date.now() / 200) * 10; // Pulsing glow
        
        ctx.fillRect(-20, line.y, ROAD_WIDTH + 40, totalHeight);
        
        // Draw checker pattern overlay
        for (let i = 0; i < line.pattern.length * 4; i++) { // Double the pattern height
            for (let j = 0; j < line.pattern[0].length * 2; j++) {
                if ((i + j) % 2 === 0) {
                    ctx.fillStyle = 'rgba(0, 0, 0, 0.8)';
                    ctx.fillRect(
                        j * (tileSize/2), 
                        line.y + i * (tileSize/2) + line.animationOffset,
                        tileSize/2,
                        tileSize/2
                    );
                }
            }
        }
        
        // Add "START" text
        ctx.font = 'bold 48px Arial';
        ctx.fillStyle = '#00FF00';
        ctx.textAlign = 'center';
        ctx.textBaseline = 'middle';
        ctx.shadowColor = '#000000';
        ctx.shadowBlur = 10;
        ctx.fillText('START', ROAD_WIDTH/2, line.y + totalHeight/2);
        
        // Add start poles with lights
        drawStartPoles(line.y, totalHeight);
    } else {
        // Draw finish line with special effects
        const gradient = ctx.createLinearGradient(0, line.y, 0, line.y + totalHeight);
        gradient.addColorStop(0, 'rgba(255, 255, 255, 0.8)');
        gradient.addColorStop(0.5, 'rgba(255, 255, 255, 1)');
        gradient.addColorStop(1, 'rgba(255, 255, 255, 0.8)');
        
        ctx.fillStyle = gradient;
        ctx.shadowColor = '#FF0000';
        ctx.shadowBlur = 20;
        
        // Draw the base line
        ctx.fillRect(-20, line.y, ROAD_WIDTH + 40, totalHeight);
        
        // Draw checker pattern overlay
        for (let i = 0; i < line.pattern.length * 2; i++) {
            for (let j = 0; j < line.pattern[0].length * 2; j++) {
                if ((i + j) % 2 === 0) {
                    ctx.fillStyle = 'rgba(0, 0, 0, 0.8)';
                    ctx.fillRect(
                        j * (tileSize/2), 
                        line.y + i * (tileSize/2) + line.animationOffset,
                        tileSize/2,
                        tileSize/2
                    );
                }
            }
        }
        
        // Add finish flags
        const flagHeight = totalHeight * 2;
        const flagWidth = 40;
        
        // Left flag
        ctx.fillStyle = '#FF0000';
        ctx.fillRect(-flagWidth, line.y - flagHeight/4, flagWidth, flagHeight);
        
        // Right flag
        ctx.fillRect(ROAD_WIDTH, line.y - flagHeight/4, flagWidth, flagHeight);
        
        // Add "FINISH" text
        ctx.font = 'bold 48px Arial';
        ctx.fillStyle = '#FF0000';
        ctx.textAlign = 'center';
        ctx.textBaseline = 'middle';
        ctx.shadowColor = '#000000';
        ctx.shadowBlur = 10;
        ctx.fillText('FINISH', ROAD_WIDTH/2, line.y + totalHeight/2);
    }
    
    ctx.restore();
}

// Add flag drawing function

// Add victory handling
function handleVictory() {
    gameOver = true;
    
    // Create victory effects
    createVictoryEffects();
    
    setTimeout(() => {
        clearInterval(gameLoop);
        gameStarted = false;
        
        if (gameOverEl) {
            gameOverEl.classList.remove('hidden');
            gameOverEl.innerHTML = `
                <h2>Victory!</h2>
                <p>Final Score: ${score}</p>
                <button id="playAgainBtn">Play Again</button>
            `;
            
            // Add event listener to the play again button
            const playAgainBtn = document.getElementById('playAgainBtn');
            if (playAgainBtn) {
                playAgainBtn.addEventListener('click', () => {
                    resetGame();
                    gameOverEl?.classList.add('hidden');
                });
            }
        }
    }, 2000);
}

// Add victory effects
function createVictoryEffects() {
    // Create celebratory particles
    for (let i = 0; i < 100; i++) {
        const angle = Math.random() * Math.PI * 2;
        const speed = 2 + Math.random() * 5;
        const color = ['#FFD700', '#FFA500', '#FF4500', '#FF0000', '#FF1493'][Math.floor(Math.random() * 5)];
        
        collisionParticles.push({
            x: playerCar.x + CAR_WIDTH / 2,
            y: playerCar.y + CAR_HEIGHT / 2,
            vx: Math.cos(angle) * speed,
            vy: Math.sin(angle) * speed - 5,
            size: 5 + Math.random() * 5,
            color: color,
            alpha: 1,
            life: 120
        });
    }
    
    // Add victory shockwave
    shockwaves.push({
        x: playerCar.x + CAR_WIDTH / 2,
        y: playerCar.y + CAR_HEIGHT / 2,
        radius: 10,
        maxRadius: 200,
        alpha: 1
    });
    
    screenShake.intensity = 10;
}

// Update startCountdown function to reset finish line properly
function startCountdown() {
    isCountingDown = true;
    countdownValue = 3;
    currentSpeed = 0;
    gameStarted = false;
    
    // Reset start line position
    startLine.y = ROAD_HEIGHT - 100;
    startLine.animationOffset = 0;
    
    // Reset finish line position
    finishLine.y = -10000; // Much longer distance
    finishLine.animationOffset = 0;
    
    // Start the game loop for visual updates during countdown
    gameLoop = window.setInterval(update, 1000/60);
    
    // Start countdown sequence
    countdownInterval = window.setInterval(() => {
        countdownValue--;
        if (countdownValue <= 0) {
            clearInterval(countdownInterval);
            isCountingDown = false;
            gameStarted = true;
        }
    }, 1000);
}

// Add start poles drawing function
function drawStartPoles(y: number, height: number) {
    const poleWidth = 30;
    const poleHeight = height * 2;
    const lightSize = 20;
    const lightSpacing = 40;
    
    // Draw poles
    ctx.fillStyle = '#333333';
    ctx.fillRect(-poleWidth, y - poleHeight/2, poleWidth, poleHeight);
    ctx.fillRect(ROAD_WIDTH, y - poleHeight/2, poleWidth, poleHeight);
    
    // Draw traffic lights
    for (let i = 0; i < 3; i++) {
        // Left pole lights
        ctx.beginPath();
        ctx.arc(-poleWidth/2, y - poleHeight/3 + i * lightSpacing, lightSize, 0, Math.PI * 2);
        ctx.fillStyle = countdownValue > i ? '#FF0000' : '#333333';
        if (!isCountingDown && gameStarted) ctx.fillStyle = '#00FF00';
        ctx.fill();
        ctx.strokeStyle = '#000000';
        ctx.lineWidth = 2;
        ctx.stroke();
        
        // Right pole lights
        ctx.beginPath();
        ctx.arc(ROAD_WIDTH + poleWidth/2, y - poleHeight/3 + i * lightSpacing, lightSize, 0, Math.PI * 2);
        ctx.fillStyle = countdownValue > i ? '#FF0000' : '#333333';
        if (!isCountingDown && gameStarted) ctx.fillStyle = '#00FF00';
        ctx.fill();
        ctx.strokeStyle = '#000000';
        ctx.lineWidth = 2;
        ctx.stroke();
    }
}

// Update drawCountdown function with improved visuals
function drawCountdown() {
    ctx.save();
    
    // Semi-transparent dark overlay
    ctx.fillStyle = 'rgba(0, 0, 0, 0.85)';
    ctx.fillRect(0, 0, canvas.width, canvas.height);
    
    // Draw main countdown with enhanced style
    ctx.font = 'bold 150px "Arial Black"';
    ctx.textAlign = 'center';
    ctx.textBaseline = 'middle';
    
    // Enhanced glow effect
    ctx.shadowColor = countdownValue > 0 ? '#FF0000' : '#00FF00';
    ctx.shadowBlur = 30;
    ctx.shadowOffsetX = 0;
    ctx.shadowOffsetY = 0;
    
    if (countdownValue > 0) {
        // Countdown numbers with gradient
        const gradient = ctx.createLinearGradient(
            canvas.width/2 - 50,
            canvas.height/2 - 60 - 50,
            canvas.width/2 + 50,
            canvas.height/2 - 60 + 50
        );
        gradient.addColorStop(0, '#FF6B6B');
        gradient.addColorStop(1, '#FF0000');
        ctx.fillStyle = gradient;
        ctx.fillText(countdownValue.toString(), canvas.width/2, canvas.height/2 - 60);
    } else {
        // GO! text with gradient
        const gradient = ctx.createLinearGradient(
            canvas.width/2 - 100,
            canvas.height/2 - 60 - 50,
            canvas.width/2 + 100,
            canvas.height/2 - 60 + 50
        );
        gradient.addColorStop(0, '#4AFF4A');
        gradient.addColorStop(1, '#00FF00');
        ctx.fillStyle = gradient;
        ctx.fillText('GO!', canvas.width/2, canvas.height/2 - 60);
    }
    
    // Draw "Get Ready!" text
    ctx.font = 'bold 36px Arial';
    ctx.fillStyle = '#FFD700';
    ctx.shadowBlur = 15;
    ctx.shadowColor = '#FF8C00';
    ctx.fillText('Get Ready!', canvas.width/2, canvas.height/2 - 160);
    
    // Draw control instructions with improved style
    const instructionStartY = canvas.height/2 + 60;
    const instructionSpacing = 45;
    
    // Instructions header
    ctx.font = 'bold 32px Arial';
    ctx.fillStyle = '#FFD700';
    ctx.shadowBlur = 15;
    ctx.shadowColor = '#FF8C00';
    ctx.fillText('HOW TO PLAY', canvas.width/2, instructionStartY - instructionSpacing);
    
    // Draw each control instruction with enhanced style
    CONTROL_INSTRUCTIONS.forEach((instruction, index) => {
        const y = instructionStartY + index * instructionSpacing;
        
        // Key box with gradient
        const boxWidth = 100;
        const boxHeight = 35;
        const boxX = canvas.width/2 - 180;
        const boxY = y - boxHeight/2;
        
        // Draw key box with gradient
        const boxGradient = ctx.createLinearGradient(boxX, boxY, boxX, boxY + boxHeight);
        boxGradient.addColorStop(0, '#4A90E2');
        boxGradient.addColorStop(1, '#357ABD');
        ctx.fillStyle = boxGradient;
        
        // Rounded box with shadow
        ctx.shadowColor = 'rgba(0, 0, 0, 0.5)';
        ctx.shadowBlur = 10;
        ctx.shadowOffsetX = 2;
        ctx.shadowOffsetY = 2;
        ctx.beginPath();
        ctx.roundRect(boxX, boxY, boxWidth, boxHeight, 8);
        ctx.fill();
        
        // Draw key text
        ctx.shadowColor = '#000';
        ctx.shadowBlur = 2;
        ctx.shadowOffsetX = 1;
        ctx.shadowOffsetY = 1;
        ctx.fillStyle = '#FFFFFF';
        ctx.font = 'bold 20px Arial';
        ctx.textAlign = 'center';
        ctx.fillText(instruction.key, boxX + boxWidth/2, y + 7);
        
        // Draw action text with icon
        ctx.textAlign = 'left';
        ctx.font = '24px Arial';
        ctx.fillStyle = '#FFFFFF';
        ctx.shadowBlur = 3;
        ctx.fillText(`${instruction.icon}  ${instruction.action}`, boxX + boxWidth + 30, y + 7);
    });
    
    // Draw difficulty level with enhanced style
    ctx.font = 'bold 24px Arial';
    ctx.textAlign = 'center';
    const difficultyGradient = ctx.createLinearGradient(
        canvas.width/2 - 100,
        instructionStartY + CONTROL_INSTRUCTIONS.length * instructionSpacing + 20,
        canvas.width/2 + 100,
        instructionStartY + CONTROL_INSTRUCTIONS.length * instructionSpacing + 40
    );
    difficultyGradient.addColorStop(0, '#FFD700');
    difficultyGradient.addColorStop(1, '#FFA500');
    ctx.fillStyle = difficultyGradient;
    ctx.shadowBlur = 10;
    ctx.shadowColor = '#FF8C00';
    ctx.fillText(
        `Difficulty: ${playerCar.label.replace('PLAYER (', '').replace(')', '')}`,
        canvas.width/2,
        instructionStartY + CONTROL_INSTRUCTIONS.length * instructionSpacing + 30
    );
    
    ctx.restore();
}

// Add CSS style to prevent canvas movement
export function addCanvasStyles() {
    if (canvas) {
        canvas.style.position = 'fixed';
        canvas.style.touchAction = 'none'; // Prevents default touch actions
        canvas.style.userSelect = 'none'; // Prevents text selection
        canvas.style.webkitUserSelect = 'none';
    }
}

// Add after spawnTrafficCar function
function spawnDevOrientationCars() {
    // Clear existing traffic
    trafficCars = [];
    
    // Spawn one of each vehicle type side by side
    const vehicleTypes: VehicleType[] = ['SPORTS', 'PICKUP', 'POLICE', 'BMW', 'BIKE'];
    const spacing = ROAD_WIDTH / (vehicleTypes.length + 1); // Even spacing across road
    
    vehicleTypes.forEach((type, index) => {
        trafficCars.push({
            x: spacing * (index + 1) - CAR_WIDTH/2, // Evenly space cars horizontally
            y: ROAD_HEIGHT/2, // All cars at same vertical position
            width: CAR_WIDTH,
            height: CAR_HEIGHT,
            speed: 0, // Static in dev mode
            color: COLORS.TRAFFIC_CARS[index % COLORS.TRAFFIC_CARS.length],
            carType: type
        });
    });
} 